#include "JobManager.h"
#include "CondorManager.h"
#define MYSQLPP_SSQLS_NO_STATICS
#include "RenderingMax.h"
#include "ConfigureManager.h"
// this is actually a simply a job status cache

using namespace CloudResourceManager;


CondorJobContextMap JobManager::condorJobCache;


CondorJobContext::CondorJobContext()
{
    startRunTime=0;
    lastCheckTime=0;
    status = JobStatusEnum::Unknown;
}

void JobManager::ClearCondorJobFromCache(const string& condorJobId)
{
    condorJobCache.erase(condorJobId);
}


bool JobManager::CheckCondorJob(const string& condorJobId, CondorJobContext& condorJobContext)
{
    bool bResult = false;
    static int NeedRefreshIntervalInSeconds = -1;
    static string CondorCheckingIntervalInSeconds;
    bool bNeedRefresh = false;

    CondorJobContextMap::iterator it;

    if ((it=condorJobCache.find(condorJobId))==condorJobCache.end())
    {
        //ConfigLogger.Instance.LogInfo("condorJobId" + condorJobId + " not exist in cache, create new instance");
        FILE_LOG(logINFO)<<"condorJobId"<<condorJobId<<" not exist in cache, create new instance";
        bNeedRefresh = true;
    }
    else
    {
        time_t nSeconds = time(NULL) - condorJobContext.lastCheckTime;
        if (NeedRefreshIntervalInSeconds == -1)
        {
            CondorCheckingIntervalInSeconds = ConfigureManager::AppSettings("CondorCheckingIntervalInSeconds");
            sscanf(CondorCheckingIntervalInSeconds.c_str(), "%d", &CondorCheckingIntervalInSeconds);
        }
        if (nSeconds > NeedRefreshIntervalInSeconds || it->second.status == JobStatusEnum::Unknown)
        {
            bNeedRefresh = true;
        }
    }

    if (bNeedRefresh)
    {
        int runtimeInSeconds = 0;
        string status;
        if ((runtimeInSeconds = CondorManager::find_job_array(condorJobId, status))!= -1)
        {
            //ConfigLogger.Instance.LogInfo("CheckCondorJob: condorJobId=" + condorJobId + "status:" + strArray[0] + " runtime:" + strArray[1]);
            FILE_LOG(logINFO)<<"CheckCondorJob: condorJobId=" <<condorJobId<< " status:" <<status
                << " runtime:" <<runtimeInSeconds;
            time_t now;
            time(&now);
            if (it == condorJobCache.end())
            {
                condorJobContext.startRunTime = now - runtimeInSeconds;
                condorJobContext.lastCheckTime = now;
                condorJobContext.status = CondorStatus2JobStatus(status);
                condorJobCache.insert(StringCondorJobContextPair(condorJobId, condorJobContext));
            }
            else
            {
                it->second.startRunTime = now - runtimeInSeconds;
                it->second.lastCheckTime = now;
                it->second.status = CondorStatus2JobStatus(status);
                condorJobContext = it->second;
            }
            bResult = true;
        }
        else
        {
            //ConfigLogger.Instance.LogError("ScriptManager.runScriptWrapperArray error for script condor_find_job_array.ps1:");
            FILE_LOG(logERROR)<<"CondorManager condor_find_job_array error";
        }
    }
    return bResult;
}

time_t JobManager::GetCondorJobRuntime(const string& condorJobId)
{
    CondorJobContext condorJobContext;
    if (CheckCondorJob(condorJobId, condorJobContext))
    {
        time_t timeSpan;
        time(&timeSpan);
        timeSpan -= condorJobContext.startRunTime;
        return timeSpan;
    }
    else
    {
        return 0;
    }
}


int JobManager::CondorStatus2JobStatus(const string& condorStatus)
{
    int myJobStatus = JobStatusEnum::Unknown;
    if (condorStatus.size()== 1)
    {
        switch (condorStatus[0])
        {
            case 'C':
                myJobStatus = JobStatusEnum::Finish;
                break;
            case 'I':
                myJobStatus = JobStatusEnum::Idle;
                break;
            case 'R':
                myJobStatus = JobStatusEnum::Running;
                break;
            case 'E':
                myJobStatus = JobStatusEnum::Error;
                break;
            case 'X':
                myJobStatus = JobStatusEnum::Abort;
                break;
            case 'U':
                myJobStatus = JobStatusEnum::Unknown;
                break;
        }
    }
    return myJobStatus;
}

int JobManager::GetCondorJobStatus(const string& condorJobId)
{
    CondorJobContext condorJobContext;

    if (CheckCondorJob(condorJobId, condorJobContext))
    {
        return condorJobContext.status;
    }
    else
    {
        return JobStatusEnum::Unknown;
    }
}
