#include "DBEnumComboBox.h"
#include "MySqlCon.h"

using namespace mysqlpp;

boost::mutex DBEnumComboBox::mutex;
bool DBEnumComboBox::bInitialized = false;
MySqlCon::StringIntMap DBEnumComboBox::nameIdMap;
std::string DBEnumComboBox::tableName;

void DBEnumComboBox::refresh()
{
    mutex.lock();
    bInitialized = false;
    mutex.unlock();
    Initialize(tableName);
}

int DBEnumComboBox::nameId(int index)
{
    return nameIdMap[itemText(index).toUTF8()];
}

int DBEnumComboBox::nameId(const std::string& name)
{
    return nameIdMap[name];
}

int DBEnumComboBox::currentNameId()
{
    return nameIdMap[currentText().toUTF8()];
}

void DBEnumComboBox::Initialize(const std::string& strTableName)
{
    mutex.lock();
    if (!bInitialized || strTableName.compare(tableName) != 0)
    {
        MySqlCon con;
        nameIdMap.clear();
        tableName = strTableName;
        if (con.GetEnumTable(strTableName, nameIdMap))
        {
            for (MySqlCon::StringIntMapIt it = nameIdMap.begin(); it != nameIdMap.end(); it ++)
            {
                addItem(it->first);
            }
        }
    }
    mutex.unlock();
}

DBEnumComboBox::DBEnumComboBox(const std::string& strTableName, WContainerWidget *parent)
    :WComboBox(parent)
{
    Initialize(strTableName);
}

