#include "SubmitWidget.h"
#include <Wt/WIntValidator>
#include <Wt/WBreak>
#include <Wt/WPanel>
#include <Wt/WTable>
#include <boost/lexical_cast.hpp>
#define MYSQLPP_SSQLS_NO_STATICS
#include "RenderingMax.h"

using namespace CloudResourceManager;

SubmitWidget::SubmitWidget(WContainerWidget* parent)
    :WContainerWidget(parent), m_parent(parent), m_notifyObj(NULL)
{
    ///////////////////////////////////
    fileTreeViewMsg = new WText("select rendering file by checkboxing");
    addWidget(fileTreeViewMsg);

    fileTreeView = new FileTreeView(this);

    WPanel* panel = new WPanel(this);
    panel->resize(600, 300);
    panel->setTitle("select rendering file:");
    panel->setCollapsible(true);

    panel->setCentralWidget(fileTreeView);

    addWidget(new WBreak());
    //////////////////////////////////
    WTable* layout =  new WTable(this);
    ////////////////////////////////////////
    layout->elementAt(1,0)->addWidget(new WText("select number of rendering nodes:"));
    layout->elementAt(1,0)->setContentAlignment(Wt::AlignRight);

    instanceNumberBox = new WSpinBox();
    instanceNumberBox->setRange(1, 10);
    instanceNumberBox->setValue(1);
    instanceNumberBox->setReadOnly(false);
    instanceNumberBox->setDisabled(false);
    layout->elementAt(1,1)->addWidget(instanceNumberBox);
    //addWidget(new WBreak());
    //////////////////////////////////
    layout->elementAt(2,0)->addWidget(new WText("select output format:"));
    layout->elementAt(2,0)->setContentAlignment(Wt::AlignRight);
    outputFormatBox = new WComboBox();
    outputFormatBox->addItem("JPEG");
    outputFormatBox->addItem("PNG");
    outputFormatBox->addItem("BMP");
    layout->elementAt(2,1)->addWidget(outputFormatBox);
    //addWidget(new WBreak());
    /////////////////////////////////
    layout->elementAt(3, 0)->addWidget(new WText("select rendering node type:"));
    layout->elementAt(3,0)->setContentAlignment(Wt::AlignRight);
    instanceTypeBox = new DBEnumComboBox("EC2_InstanceTypes");
    layout->elementAt(3,1)->addWidget(instanceTypeBox);
    //    addWidget(new WBreak());
    ////////////////////////////////
    layout->elementAt(4, 0)->addWidget(new WText("select starting rendering frame:"));
    layout->elementAt(4,0)->setContentAlignment(Wt::AlignRight);
    startFrameBox = new WSpinBox();
    startFrameBox->setMinimum(1);
    startFrameBox->setValue(1);
    startFrameBox->setReadOnly(false);
    startFrameBox->setDisabled(false);
    layout->elementAt(4,1)->addWidget(startFrameBox);
    //addWidget(new WBreak());
    ///////////////////////////////
    layout->elementAt(5,0)->addWidget(new WText("select ending rendering frame:"));
    layout->elementAt(5,0)->setContentAlignment(Wt::AlignRight);
    endFrameBox = new WSpinBox();
    endFrameBox->setMinimum(1);
    endFrameBox->setValue(1);
    endFrameBox->setReadOnly(false);
    endFrameBox->setDisabled(false);
    layout->elementAt(5,1)->addWidget(endFrameBox);
    endFrameMsg = new WText("cannot be less than starting frame number!");
    endFrameMsg->decorationStyle().setForegroundColor(Wt::red);
    layout->elementAt(5,2)->addWidget(endFrameMsg);
    //addWidget(new WBreak());
    ///////////////////////////////////////////////////
    //addWidget(new WBreak());

    msg = new WText("debug msg:");
    msg->decorationStyle().setForegroundColor(Wt::blue);
    msg->decorationStyle().setTextDecoration(WCssDecorationStyle::Overline);
    layout->elementAt(6, 0)->addWidget(msg);
    layout->elementAt(6, 0)->setColumnSpan(2);
    layout->elementAt(6, 0)->setRowSpan(6);
    //////////////////////////////////
    submitButton = new WPushButton("submit", layout->elementAt(13,0));
    layout->elementAt(13,0)->setContentAlignment(Wt::AlignRight);

    submitButton->clicked().connect(this, &SubmitWidget::submit);
    summaryButton = new WPushButton("summary", layout->elementAt(13,1));
    layout->elementAt(13,1)->setContentAlignment(Wt::AlignLeft);
    summaryButton->clicked().connect(this, &SubmitWidget::summary);

    submitButton->setEnabled(false);

    fileTreeView->changed().connect(submitButton, &WPushButton::disable);
    instanceNumberBox->valueChanged().connect(submitButton, &WPushButton::disable);
    outputFormatBox->changed().connect(submitButton, &WPushButton::disable);
    instanceTypeBox->changed().connect(submitButton, &WPushButton::disable);
    startFrameBox->valueChanged().connect(submitButton, &WPushButton::disable);
    endFrameBox->valueChanged().connect(submitButton, &WPushButton::disable);


    fileTreeView->changed().connect(summaryButton, &WPushButton::enable);
    instanceNumberBox->valueChanged().connect(summaryButton, &WPushButton::enable);
    outputFormatBox->changed().connect(summaryButton, &WPushButton::enable);
    instanceTypeBox->changed().connect(summaryButton, &WPushButton::enable);
    startFrameBox->valueChanged().connect(summaryButton, &WPushButton::enable);
    endFrameBox->valueChanged().connect(summaryButton, &WPushButton::enable);

    //endFrameBox->valueChanged().connect(this, &SubmitWidget::validateFrameNumber);
}

void SubmitWidget::setFolderName(const std::string& strFolderName)
{
    m_folderName =  strFolderName;
    fileTreeView->setPath(m_folderName);
}

void SubmitWidget::setupNotifyEventMethod(NotifyMethod notifyMethod, WObject* notifyObj)
{
    m_notifyMethod = notifyMethod;
    m_notifyObj = notifyObj;
}

bool SubmitWidget::doSubmit()
{
    MySqlppHelper mySqlppHelper;
    Tasks myTask;
    int taskId=0;
    myTask.UserId = m_userId;
    myTask.TaskTypeId = TaskTypeEnum::Blender_Rendering;
    myTask.TaskStatusId = TaskStatusEnum::New;
    std::string param;
    param = boost::lexical_cast<std::string>(startFrameBox->value());
    param += ":" + boost::lexical_cast<std::string>(endFrameBox->value());
    param += ":" + outputFormatBox->currentText().toUTF8();
    myTask.TaskInputParameter = param;
    myTask.TaskInputFolder = m_folderName;

    std::string checkedFile;
    fileTreeView->getCheckedFile(checkedFile);
    myTask.TaskMainFile = checkedFile;
    myTask.MaxInstanceNumber = instanceNumberBox->value();

    myTask.SpotInstance = true;
    myTask.EC2InstanceTypeId = instanceTypeBox->currentNameId();
    myTask.LastCheckTime = mysqlpp::sql_datetime(time(NULL));
    //myTask.EC2SnapshotId= "EC2SnapshotId ";
    myTask.TaskDesc = "TaskDesc";
    myTask.TaskComment = "TaskComment";
    myTask.CreatedBy = m_userId;
    myTask.ModifiedOn = mysqlpp::sql_datetime(time(NULL));
    myTask.ModifiedBy = m_userId;
    myTask.NotificationTypeId = NotificationTypeEnum::Email;

    return mySqlppHelper.SaveNewTasks(myTask) != 0;
}

void SubmitWidget::submit()
{
    bool bSubmitResult = true;
    std::string str;
    bSubmitResult = doSubmit();
    str = "submit ";
    str += bSubmitResult? "successful!" : "failed!";
    msg->setText(str);
    submitButton->setEnabled(false);
    if (m_notifyObj)
    {
        boost::bind<void>(m_notifyMethod, m_notifyObj, _1)(bSubmitResult ? SubmitSuccessful:SubmitFailed);
    }
}

void SubmitWidget::summary()
{
    if (endFrameBox->value() < startFrameBox->value())
    {
        endFrameMsg->decorationStyle().setForegroundColor(Wt::red);
        endFrameMsg->decorationStyle().setTextDecoration(WCssDecorationStyle::Blink);
        return;
    }
    else
    {
        endFrameMsg->decorationStyle().setForegroundColor(Wt::blue);
        endFrameMsg->decorationStyle().setTextDecoration(WCssDecorationStyle::LineThrough);
    }

    std::string path;
    if (!fileTreeView->getCheckedFile(path))
    {
        fileTreeViewMsg->decorationStyle().setForegroundColor(Wt::red);
        fileTreeViewMsg->decorationStyle().setTextDecoration(WCssDecorationStyle::Blink);
        return;
    }
    else
    {
        fileTreeViewMsg->decorationStyle().setForegroundColor(Wt::blue);
        fileTreeViewMsg->decorationStyle().setTextDecoration(WCssDecorationStyle::LineThrough);
    }


    std::string str;

    str += "<p>RenderFile:<b>" + path + "</b></p>";

    str += "<p>InstanceNumber:<b>" + boost::lexical_cast<std::string>(instanceNumberBox->value()) + "</b></p>";
    str += "<p>OutputFormat:<b>" + outputFormatBox->currentText().toUTF8() + "</b></p>";
    str += "<p>InstanceType:<b>" + instanceTypeBox->currentText().toUTF8() + "</b></p>";
    str += "<p>StartFrame:<b>" + boost::lexical_cast<std::string>(startFrameBox->value()) + "</b></p>";
    str += "<p>EndFrame:<b>" + boost::lexical_cast<std::string>(endFrameBox->value()) + "</b></p>";
    str += "To proceed, click submit button";
    msg->setText(str);
    submitButton->setEnabled(true);
}








