#include <Wt/WApplication>
#include <Wt/WBreak>
#include <Wt/WContainerWidget>
#include <Wt/WLineEdit>
#include <Wt/WPushButton>
#include <Wt/WText>
#include <Wt/WLabel>
#include <Wt/WMessageBox>
#include <Wt/WRegExpValidator>
#include "LoginWidget.h"
#include "RegisterWidget.h"
#include "SubmitWidget.h"
#include <boost/regex.hpp>
#include "DBEnumComboBox.h"
#include <Wt/FileTreeList.h>
#include "DBTableWidget.h"
#include "BasicControls.h"
#include "MainMenuWidget.h"
#include "UploadFileWidget.h"
#include "TaskViewWidget.h"
#include "SessionManagerWidget.h"
#include <Wt/WValidator>


using namespace Wt;

class HelloApplication : public WApplication
{
private:
    LoginWidget* login;
    RegisterWidget* regist;
    bool validPass(const std::string& user, const std::string& pass)
    {
        WMessageBox::show(user, pass, Ok);
        return true;
    }

    void notifyEvent(NotifyEvent event)
    {
        WMessageBox::show("notify", boost::lexical_cast<std::string>(event), Ok);
        if (event == LoginSuccessful)
        {
            login->hide();
        }
    }
    WLineEdit* input;
    WText*     msg;
    WRegExpValidator* emailRegExp;
    void myValidate()
    {
        std::string txt;
        WString str = input->text();
        if (emailRegExp->validate(str) == WValidator::Valid)
        {
            txt = "valid";
        }
        else
        {
            txt = "Invalid";
        }
        msg->setText(txt);
        input->validate();
    }
    void emailTest()
    {
        msg = new WText(root());
        input = new WLineEdit(root());

        emailRegExp = new WRegExpValidator(this);
        //emailRegExp->setRegExp("\b[A-Z0-9._%-]+@[A-Z0-9.-]+\.[A-Z]{2,4}\b");
        //emailRegExp->setRegExp("[A-Za-z0-9]+[A-Za-z0-9._\-]*@[A-Za-z0-9]+[A-Za-z0-9._\-]*\."
        //    "(([0-9]{1,3})|([a-zA-Z]{2,3})|(aero|coop|info|museum|name))");
        /*
        emailRegExp->setRegExp("([a-z0-9A-Z]([a-zA-Z0-9\._\-]{0, 10})[a-zA-Z0-9])"
            "@"
            "([A-Za-z0-9]([A-Za-z0-9\_\-]{0,10})[A-Za-z0-9])"
            "\."
            "(com|edu|net|org|([A-Za-z]{2,3}))");
            */
        emailRegExp->setRegExp(
            "([a-z0-9A-Z])"
            "(("
            "([\._-]?)"
            "([a-zA-Z0-9]+)"
            ")*)"
            "[@]"

            "([A-Za-z0-9])"
            "(("
            "([_\.-]?)"
            "([a-z0-9A-Z]+)"
            ")*)"
            "[\.]"
            "(com|edu|net|org|info|tv|([A-Za-z]{2,4}))"

            );
        emailRegExp->setMandatory(true);
        input->setValidator(emailRegExp);
        input->keyWentUp().connect(this, &HelloApplication::myValidate);
        input->clicked().connect(this, &HelloApplication::myValidate);

    }
    void checkReg()
    {
        boost::regex reg;
        reg.assign(//"([a-z0-9A-Z]([a-zA-Z0-9\._\-]{0, 10})[a-zA-Z0-9])"
            //"@"
            //"([A-Za-z0-9]([A-Za-z0-9\_\-]{0,1})[A-Za-z0-9])"
            "[\.][A-Za-z0-9]{2,3}?"
            //"(com|edu|net|org|([A-Za-z]{2,3}))");
            );
        bool result = boost::regex_match(input->text().toUTF8(), reg);
        WMessageBox::show(input->text(), boost::lexical_cast<std::string>(result), Ok);
    }

    void regTest()
    {
        input = new WLineEdit(root());
        WPushButton* checkButton = new WPushButton("check", root());
        checkButton->clicked().connect(this, &HelloApplication::checkReg);
    }

    DBEnumComboBox* dbBox;
    WText* status;
    void checkComboBox()
    {
        std::string msg;
        for (int i = 0; i < dbBox->count(); i ++)
        {
            msg += " name:" + dbBox->itemText(i).toUTF8();
            msg += " id:" + boost::lexical_cast<std::string>(dbBox->nameId(i));
        }
        status->setText(msg);
    }

    void comboTest()
    {
        dbBox = new DBEnumComboBox("JobTypes", root());

        WPushButton* checkBox = new WPushButton("check", root());
        status = new WText("status", root());

        checkBox->clicked().connect(this, &HelloApplication::checkComboBox);
    }
    MainMenuWidget* mainMenu;
    void populateMenu()
    {
        WPushButton* selectButton = new WPushButton("select", root());
        input = new WLineEdit("select", root());
        selectButton->clicked().connect(this, &HelloApplication::selectMenu);
        comboBox = new WComboBox(root());
        comboBox->addItem("true");
        comboBox->addItem("false");


        mainMenu = new MainMenuWidget(root());
        BasicControls basicControls;
        WidgetNamePairVector widgetNameVector;
        basicControls.createWidgetNameVector(widgetNameVector);
        for (int i = 0; i < widgetNameVector.size(); i ++)
        {
            mainMenu->addItem(widgetNameVector[i].first, widgetNameVector[i].second);
        }
    }
    void selectMenu()
    {
        if (comboBox->currentText().toUTF8().compare("true") == 0)
        {
            mainMenu->setDisabledByName(input->text().toUTF8(), true);
        }
        else
        {
            mainMenu->setDisabledByName(input->text().toUTF8(), false);
        }
    }
    WComboBox* comboBox;
    void mainMenuTest()
    {
        mainMenu = new MainMenuWidget(root());
        mainMenu->addItem("upload", new UploadFileWidget());
        mainMenu->addItem("register", new RegisterWidget());
        mainMenu->addItem("login", new LoginWidget());
        mainMenu->addItem("task", new TaskViewWidget());
        mainMenu->addItem("submit", new SubmitWidget());
    }

public:
    HelloApplication(const WEnvironment& env)
        : WApplication(env)
    {
        WApplication::instance()->useStyleSheet("icons/wt.css");
        /*
        login = new LoginWidget(root());
        login->setupValidLoginMethod((LoginWidget::ValidLoginMethod)(&HelloApplication::validPass), this);
        login->setupNotifyEventMethod((LoginWidget::NotifyMethod)(&HelloApplication::notifyEvent), this);
        */
        //regist = new RegisterWidget(root());
        //comboTest();
        //FileTreeList* treeList =  new FileTreeList("/home/nick/myprojects/wt-3.1.9/hello", root());
        //SubmitWidget* submitWidget = new SubmitWidget("/home/nick/myprojects/wt-3.1.9/hello", root());
        //DBTableWidget* tableWidget = new DBTableWidget("EC2_InstanceTypes", root());

        //populateMenu();
        //UploadFileWidget* uploadFileWidget = new UploadFileWidget(root());

        //TaskViewWidget* taskView = new TaskViewWidget(1, root());
        //mainMenuTest();
        emailTest();
        //regTest();

        /*
        input = new WLineEdit("", root());
        WPushButton* checkButton = new WPushButton("check", root());
        checkButton->clicked().connect(this, &HelloApplication::checkReg);
        */
    }

};

WApplication *createApplication(const WEnvironment& env)
{

    return new SessionManagerWidget(env);
    //return new HelloApplication(env);
}

int main(int argc, char **argv)
{
    return WRun(argc, argv, &createApplication);
}
